package com.kalyshev.yan.computer.service;

import com.kalyshev.yan.cabinet.model.Cabinet;
import com.kalyshev.yan.computer.model.Computer;
import com.kalyshev.yan.computer.repository.ComputerNotFoundException;
import com.kalyshev.yan.computer.repository.ComputerRepository;
import com.kalyshev.yan.monitor.model.Monitor;
import com.kalyshev.yan.monitor.service.MonitorService;
import com.kalyshev.yan.util.validation.ValidatorUtil;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Optional;

@Service
public class ComputerService {
    private final ComputerRepository computerRepository;
    private final MonitorService monitorService;
    private final ValidatorUtil validatorUtil;
    public ComputerService(ComputerRepository computerRepository,
                           MonitorService monitorService,
                           ValidatorUtil validatorUtil) {
        this.computerRepository = computerRepository;
        this.monitorService = monitorService;
        this.validatorUtil = validatorUtil;
    }
    @Transactional
    public Computer addComputer(String modelName, String serialNum, @Nullable Long monitorId) {
        if (!StringUtils.hasText(modelName)) {
            throw new IllegalArgumentException("Computer model name is null or empty");
        }
        if (!StringUtils.hasText(serialNum)) {
            throw new IllegalArgumentException("Computer serial number is null or empty");
        }
        final Computer computer = new Computer(modelName, serialNum);
        if (monitorId != null) {
            final Monitor monitor = monitorService.findMonitor(monitorId);
            computer.setMonitor(monitor);
        }
        validatorUtil.validate(computer);
        return computerRepository.save(computer);
    }
    @Transactional(readOnly = true)
    public Computer findComputer(Long id) {
        final Optional<Computer> computer = computerRepository.findById(id);
        return computer.orElseThrow(() -> new ComputerNotFoundException(id));
    }
    @Transactional(readOnly = true)
    public List<Computer> findAllComputers() {
        return computerRepository.findAll();
    }
    @Transactional(readOnly = true)
    public List<Computer> findFilteredComputers(Long id, String modelName, String serialNum, Long monitorId, Long cabinetId) {
        return computerRepository.findFilteredComputers(id, modelName, serialNum, monitorId, cabinetId);
    }
    @Transactional
    public Computer updateComputer(Long id, String modelName, String serialNum, Long monitorId, Long cabinetId) {
        if (!(StringUtils.hasText(modelName) || StringUtils.hasText(serialNum))) {
            throw new IllegalArgumentException("Need at least one argument");
        }
        final Computer currentComputer = findComputer(id);
        if (modelName != null) {
            currentComputer.setModelName(modelName);
        }
        if (serialNum != null) {
            currentComputer.setSerialNum(serialNum);
        }
        if (monitorId != null) {
            final Monitor monitor = monitorService.findMonitor(monitorId);
            currentComputer.setMonitor(monitor);
        }
        validatorUtil.validate(currentComputer);
        return computerRepository.save(currentComputer);
    }
    @Transactional
    public Computer deleteComputer(Long id) {
        final Computer currentComputer = findComputer(id);
        Cabinet currentComputerCabinet = currentComputer.getCabinet();
        if (currentComputerCabinet != null) {
            currentComputerCabinet.removeComputer(currentComputer);
        }
        computerRepository.delete(currentComputer);
        return currentComputer;
    }
    @Transactional
    public void deleteAllComputers() {
        List<Computer> computers = findAllComputers();
        for (Computer computer : computers) {
            deleteComputer(computer.getId());
        }
    }
    @Transactional
    public Monitor setMonitor(Long monitorId, Long computerId) {
        if (computerId == null) {
            throw new IllegalArgumentException("Computer id is null or empty");
        }
        if (monitorId == null) {
            throw new IllegalArgumentException("Monitor id is null or empty");
        }
        final Computer computer = findComputer(computerId);
        final Monitor monitor = monitorService.findMonitor(monitorId);
        computer.setMonitor(monitor);
        return monitor;
    }
    @Transactional
    public void deleteRelationsWithCabinets(List<Computer> computers) {
        for (Computer computer: computers) {
            computer.setCabinet(null);
        }
    }
    @Transactional
    public Computer findComputerByMonitor(Monitor monitor) {
        if (monitor.getId() == null) {
            return null;
        }
        return computerRepository.findComputerByMonitor(monitor.getId());
    }
    @Transactional
    public Monitor deleteMonitorWithRelation(Long id) {
        final Monitor currentMonitor = monitorService.findMonitor(id);
        Computer computer = findComputerByMonitor(currentMonitor);
        if (computer != null) {
            computer.removeMonitor();
        }
        monitorService.deleteMonitor(currentMonitor);
        return currentMonitor;
    }
    @Transactional
    public void deleteAllMonitorsWithRelations() {
        computerRepository.removeAllComputerMonitorRelations();
        monitorService.deleteAllMonitors();
    }
}